const { parsers: babelParsers } = require("prettier/parser-babel")

function lexicalSort(a, b) {
  if (a > b) {
    return 1
  }
  if (a < b) {
    return -1
  }
  return 0
}

function removeDuplicateKeys(obj, options = { enableSortJSON: true }) {
  if (typeof obj !== "object" || obj === null) {
    return obj
  }

  // 如果是数组，递归处理每个元素
  if (Array.isArray(obj)) {
    return obj.map((item) => removeDuplicateKeys(item, options))
  }

  // 处理对象
  const result = {}
  const keys = Object.keys(obj)
  
  // 从后向前遍历，保留最后出现的键值对
  for (let i = keys.length - 1; i >= 0; i--) {
    const key = keys[i]
    if (!(key in result)) {
      result[key] = removeDuplicateKeys(obj[key], options)
    }
  }

  // 返回一个新对象，根据选项决定是否按字母顺序排序
  const resultKeys = options.enableSortJSON ? Object.keys(result).sort(lexicalSort) : Object.keys(result)
  return resultKeys.reduce((acc, key) => {
    acc[key] = result[key]
    return acc
  }, {})
}

const parser = {
  ...babelParsers.json,
  parse: (text, parsers, options) => {
    const ast = babelParsers.json.parse(text, parsers, options)
    return {
      ...ast,
      parsed: JSON.parse(text)
    }
  },
  astFormat: "json-no-duplicates-ast"
}

const printer = {
  print: (path, options) => {
    const node = path.getValue()
    if (!node.parsed) {
      return JSON.stringify(node, null, options.tabWidth)
    }
    const deduplicated = removeDuplicateKeys(node.parsed)
    return JSON.stringify(deduplicated, null, options.tabWidth)
  },
  embed: false
}

module.exports = {
  languages: [
    {
      name: "JSON No Duplicates",
      parsers: ["json-no-duplicates"],
      extensions: [".json"],
      filenames: ["package.json", "composer.json"],
      linguistLanguageId: 174,
      vscodeLanguageIds: ["json", "jsonc"]
    }
  ],
  parsers: {
    "json-no-duplicates": {
      parse: (text, options = {}) => {
        try {
          const parsed = JSON.parse(text)
          const deduplicated = removeDuplicateKeys(parsed, { 
            enableSortJSON: options.enableSortJSON !== false // 默认为 true
          })
          return deduplicated
        } catch (error) {
          throw new Error(`Failed to parse JSON: ${error.message}`)
        }
      },
      astFormat: "json-no-duplicates-ast",
      locStart: () => 0,
      locEnd: () => 0
    }
  },
  printers: {
    "json-no-duplicates-ast": printer
  },
  options: {
    enableSortJSON: {
      type: "boolean",
      category: "Global",
      default: true,
      description: "Sort JSON object keys lexically"
    }
  },
  defaultOptions: {
    tabWidth: 2,
    enableSortJSON: true
  }
}
